==> libosl_0.3.0.orig.tar.gz: libosl_0.3.0.orig/include/osl/eval/ml/pin.h <==
/* pin.h
 */

#ifndef EVAL_ML_PIN_H
#define EVAL_ML_PIN_H

#include "osl/ptype.h"
#include "osl/misc/carray.h"
#include "osl/state/numEffectState.h"
#include "osl/eval/ml/weights.h"
#include "osl/eval/ml/midgame.h"

namespace osl
{
  namespace eval
  {
    namespace ml
    {
      class SimplePin
      {
	static CArray<int, PTYPE_SIZE> table;
      public:
	SimplePin() { };
	static void setUp(const Weights &weights);
	int eval(const NumEffectState &state,
		 PieceMask black_mask, PieceMask white_mask) const;
      };

      class Pin
      {
	static int index(const Position king,
			 const Piece piece)
	{
	  return std::abs(piece.position().x() - king.x()) * 17 +
	    (piece.owner() == BLACK ? (king.y() - piece.position().y()) :
	     (piece.position().y() - king.y())) + 8;
	}
	static CArray2d<MultiInt, PTYPE_SIZE, 17 * 9> table;
      public:
	enum { DIM = (osl::PTYPE_MAX - osl::PTYPE_PIECE_MIN + 1) * 17 * 9};
	Pin() { };
	static void setUp(const Weights &weights,int stage);
	static MultiInt eval(const NumEffectState &state,
			      PieceMask black_mask, PieceMask white_mask);
      };

      class PinPtypeAll
      {
      public:
	static MultiInt eval(const NumEffectState &state);
      private:
	template <Player Defense>
	static MultiInt evalOne(const NumEffectState &state);
	template <Player Defense>
	static bool pawnAttack(const NumEffectState &state, Piece piece)
	{
	  const Position up =
	    piece.position() + DirectionPlayerTraits<U, Defense>::offset();
	  return (up.isOnBoard() &&
		  (state.hasEffectByPtypeStrict<PAWN>(
		    PlayerTraits<Defense>::opponent,
		    up) ||
		   (!state.isPawnMaskSet(PlayerTraits<Defense>::opponent,
					 piece.position().x()) &&
		    state.getPieceAt(up).isEmpty())));
	}
      protected:
	static CArray<MultiInt, 80> table;
	static CArray<MultiInt, 48> pawn_table;
	static CArray<MultiInt, 560> distance_table;
      };

      class PinPtype : public PinPtypeAll
      {
      public:
	enum { ONE_DIM = 80, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };

      class PinPtypeDistance : public PinPtypeAll
      {
      public:
	enum { ONE_DIM = 560, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };

      class PinPtypePawnAttack : public PinPtypeAll
      {
      public:
	enum { ONE_DIM = 48, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };
    }
  }
}
#endif // EVAL_ML_PIN_H
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:

==> libosl_0.4.0.orig.tar.gz: libosl_0.4.0.orig/include/osl/eval/ml/pin.h <==
/* pin.h
 */

#ifndef EVAL_ML_PIN_H
#define EVAL_ML_PIN_H

#include "osl/ptype.h"
#include "osl/misc/carray.h"
#include "osl/state/numEffectState.h"
#include "osl/eval/ml/weights.h"
#include "osl/eval/ml/midgame.h"

namespace osl
{
  namespace eval
  {
    namespace ml
    {
      class SimplePin
      {
	static CArray<int, PTYPE_SIZE> table;
      public:
	SimplePin() { };
	static void setUp(const Weights &weights);
	int eval(const NumEffectState &state,
		 PieceMask black_mask, PieceMask white_mask) const;
      };

      class Pin
      {
	static int index(const Square king,
			 const Piece piece)
	{
	  return std::abs(piece.square().x() - king.x()) * 17 +
	    (piece.owner() == BLACK ? (king.y() - piece.square().y()) :
	     (piece.square().y() - king.y())) + 8;
	}
	static CArray2d<MultiInt, PTYPE_SIZE, 17 * 9> table;
      public:
	enum { DIM = (osl::PTYPE_MAX - osl::PTYPE_PIECE_MIN + 1) * 17 * 9};
	Pin() { };
	static void setUp(const Weights &weights,int stage);
	static MultiInt eval(const NumEffectState &state,
			      PieceMask black_mask, PieceMask white_mask);
      };

      class PinPtypeAll
      {
      public:
	static MultiInt eval(const NumEffectState &state);
      private:
	template <Player Defense>
	static MultiInt evalOne(const NumEffectState &state);
	template <Player Defense>
	static bool pawnAttack(const NumEffectState &state, Piece piece)
	{
	  const Square up =
	    piece.square() + DirectionPlayerTraits<U, Defense>::offset();
	  return (up.isOnBoard() &&
		  (state.hasEffectByPtypeStrict<PAWN>(alt(Defense), up) 
		   || (!state.isPawnMaskSet(alt(Defense),
					    piece.square().x()) 
		       && state.pieceAt(up).isEmpty())));
	}
      protected:
	static CArray<MultiInt, 80> table;
	static CArray<MultiInt, 48> pawn_table;
	static CArray<MultiInt, 560> distance_table;
      };

      class PinPtype : public PinPtypeAll
      {
      public:
	enum { ONE_DIM = 80, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };

      class PinPtypeDistance : public PinPtypeAll
      {
      public:
	enum { ONE_DIM = 560, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };

      class PinPtypePawnAttack : public PinPtypeAll
      {
      public:
	enum { ONE_DIM = 48, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };
    }
  }
}
#endif // EVAL_ML_PIN_H
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
